@@ -15,20 +15,19 @@
 
 """Thread-safe connection pool for python-memcached."""
 
-# NOTE(yorik-sar): this file is copied between keystone and keystonemiddleware
-# and should be kept in sync until we can use external library for this.
-
 import collections
 import contextlib
 import itertools
-import logging
+import queue
 import threading
 import time
 
+try:
+    import eventlet
+except ImportError:
+    eventlet = None
 import memcache
 from oslo_log import log
-from six.moves import queue
-from six.moves import zip
 
 from oslo_cache._i18n import _
 from oslo_cache import exception
@@ -46,9 +45,17 @@ class _MemcacheClient(memcache.Client):
     """
     __delattr__ = object.__delattr__
     __getattribute__ = object.__getattribute__
-    __new__ = object.__new__
     __setattr__ = object.__setattr__
 
+    # Hack for lp 1812935
+    if eventlet and eventlet.patcher.is_monkey_patched('thread'):
+        # NOTE(bnemec): I'm not entirely sure why this works in a
+        # monkey-patched environment and not with vanilla stdlib, but it does.
+        def __new__(cls, *args, **kwargs):
+            return object.__new__(cls)
+    else:
+        __new__ = object.__new__
+
     def __del__(self):
         pass
 
@@ -116,7 +123,7 @@ class ConnectionPool(queue.Queue):
             LOG.log(level, prefix + msg, *args, **kwargs)
 
     def _debug_logger(self, msg, *args, **kwargs):
-        self._do_log(logging.DEBUG, msg, *args, **kwargs)
+        self._do_log(log.DEBUG, msg, *args, **kwargs)
 
     def _trace_logger(self, msg, *args, **kwargs):
         self._do_log(log.TRACE, msg, *args, **kwargs)
@@ -124,6 +131,7 @@ class ConnectionPool(queue.Queue):
     @contextlib.contextmanager
     def acquire(self):
         self._trace_logger('Acquiring connection')
+        self._drop_expired_connections()
         try:
             conn = self.get(timeout=self._connection_get_timeout)
         except queue.Empty:
@@ -136,7 +144,6 @@ class ConnectionPool(queue.Queue):
             yield conn
         finally:
             self._trace_logger('Releasing connection %s', id(conn))
-            self._drop_expired_connections()
             try:
                 # super() cannot be used here because Queue in stdlib is an
                 # old-style class
@@ -204,7 +211,27 @@ class MemcacheClientPool(ConnectionPool):
         self._hosts_deaduntil = [0] * len(urls)
 
     def _create_connection(self):
-        return _MemcacheClient(self.urls, **self._arguments)
+        # NOTE(morgan): Explicitly set flush_on_reconnect for pooled
+        # connections. This should ensure that stale data is never consumed
+        # from a server that pops in/out due to a network partition
+        # or disconnect.
+        #
+        # See the help from python-memcached:
+        #
+        # param flush_on_reconnect: optional flag which prevents a
+        #        scenario that can cause stale data to be read: If there's more
+        #        than one memcached server and the connection to one is
+        #        interrupted, keys that mapped to that server will get
+        #        reassigned to another. If the first server comes back, those
+        #        keys will map to it again. If it still has its data, get()s
+        #        can read stale data that was overwritten on another
+        #        server. This flag is off by default for backwards
+        #        compatibility.
+        #
+        # The normal non-pooled clients connect explicitly on each use and
+        # does not need the explicit flush_on_reconnect
+        return _MemcacheClient(self.urls, flush_on_reconnect=True,
+                               **self._arguments)
 
     def _destroy_connection(self, conn):
         conn.disconnect_all()
